var path = require('path');
var Project = require('../lib/project');
var fs = require('fs');
var _ = require('underscore');
var args = require('optimist').argv;

Meteorite = function() {
  // Make directories we'll need
  Meteorite.prepareFS();
  
  this.appName = args._[1];
  this.project = new Project();
  this.defaultSmartJson = { packages: {} };
  this.appPath = path.join(process.cwd(), this.appName);
  this.smartJsonPath = path.join(this.appPath, 'smart.json');
};

// Meteorite command
Meteorite.prototype.create = function(fn) {
  var self = this;

  // Make sure we have an app
  if (this.appName) {
    
    this.project.execute(process.argv.slice(2), function() {
      // New project needs a new smart json file
      self.installDefaultSmartJson(fn);
      
      // TODO -- do we want to run project.install too?
    });
    
  }
};

Meteorite.prototype.install = function(fn) {
  this.project.install(fn);
};

Meteorite.prototype.uninstall = function(fn) {
  this.project.uninstall(fn);
};

// Meteor commands, will get run either by the project install or the default meteor
// FIXME -- 'update'?
// TODO -- treat add, remove, list specially
_.each(['run', 'add', 'remove', 'list', 'bundle', 'mongo', 'deploy', 'logs', 'reset'], function(command) {
  Meteorite.prototype[command] = function(fn) {
    this.project.execute(process.argv.slice(2), fn);
  }
});


Meteorite.prototype.installDefaultSmartJson = function(fn) {

  // Don't do it if there's already one
  if (path.existsSync(this.smartJsonPath))
    return fn();

  // Make a nicely formated default json string
  var smartJsonString = JSON.stringify(this.defaultSmartJson, null, 2) + "\n";

  // Install it into new project
  fs.writeFile(this.smartJsonPath, smartJsonString, function(err) {
    if (err) {
      console.log("Error: could not create smart.json", err);
      fn();
    } else {
      fn();
    }
  });

};

// Class methods

Meteorite.root = function() {
  var homeDir = process.env.HOME;
  return path.join(homeDir, '.meteorite');
};

// Creates the path to ~/.meteorite
Meteorite.prepareFS = function() {
  var root = Meteorite.root();
  if (!path.existsSync(root))
    fs.mkdir(root);
};

module.exports = Meteorite;
